@@ -97,10 +97,6 @@ char buffer[PATH_MAX * 2];
 #define OPTION_HELP                 "--help"
 #define OPTION_OUT_FMT_IHX          "--out-fmt-ihx"
 #define OPTION_OUT_FMT_S19          "--out-fmt-s19"
-#define OPTION_HUGE_MODEL           "--model-huge"
-#define OPTION_LARGE_MODEL          "--model-large"
-#define OPTION_MEDIUM_MODEL         "--model-medium"
-#define OPTION_SMALL_MODEL          "--model-small"
 #define OPTION_PEEP_FILE            "--peep-file"
 #define OPTION_LIB_PATH             "--lib-path"
 #define OPTION_CALLEE_SAVES         "--callee-saves"
@@ -136,9 +132,11 @@ char buffer[PATH_MAX * 2];
 #define OPTION_STD_C95              "--std-c95"
 #define OPTION_STD_C99              "--std-c99"
 #define OPTION_STD_C11              "--std-c11"
+#define OPTION_STD_C2X              "--std-c2x"
 #define OPTION_STD_SDCC89           "--std-sdcc89"
 #define OPTION_STD_SDCC99           "--std-sdcc99"
 #define OPTION_STD_SDCC11           "--std-sdcc11"
+#define OPTION_STD_SDCC2X           "--std-sdcc2x"
 #define OPTION_CODE_SEG             "--codeseg"
 #define OPTION_CONST_SEG            "--constseg"
 #define OPTION_DATA_SEG             "--dataseg"
@@ -155,6 +153,11 @@ char buffer[PATH_MAX * 2];
 #define OPTION_DUMP_I_CODE          "--dump-i-code"
 #define OPTION_DUMP_GRAPHS          "--dump-graphs"
 
+#define OPTION_SMALL_MODEL          "--model-small"
+#define OPTION_MEDIUM_MODEL         "--model-medium"
+#define OPTION_LARGE_MODEL          "--model-large"
+#define OPTION_HUGE_MODEL           "--model-huge"
+
 static const OPTION optionsTable[] = {
   {0,   NULL, NULL, "General options"},
   {0,   OPTION_HELP, NULL, "Display this help"},
@@ -190,6 +193,8 @@ static const OPTION optionsTable[] = {
   {0,   OPTION_STD_SDCC99, NULL, "Use ISO C99 standard with SDCC extensions"},
   {0,   OPTION_STD_C11, NULL, "Use ISO C11 standard (incomplete)"},
   {0,   OPTION_STD_SDCC11, NULL, "Use ISO C11 standard with SDCC extensions (default)"},
+  {0,   OPTION_STD_C2X, NULL, "Use ISO C2X standard (incomplete)"},
+  {0,   OPTION_STD_SDCC2X, NULL, "Use ISO C2X standard with SDCC extensions"},
   {0,   OPTION_DOLLARS_IN_IDENT, &options.dollars_in_ident, "Permit '$' as an identifier character"},
   {0,   OPTION_SIGNED_CHAR, &options.signed_char, "Make \"char\" signed by default"},
   {0,   OPTION_USE_NON_FREE, &options.use_non_free, "Search / include non-free licensed libraries and header files"},
@@ -197,10 +202,6 @@ static const OPTION optionsTable[] = {
   {0,   NULL, NULL, "Code generation options"},
   {'m', NULL, NULL, "Set the port to use e.g. -mz80."},
   {'p', NULL, NULL, "Select port specific processor e.g. -mpic14 -p16f84"},
-  {0,   OPTION_SMALL_MODEL, NULL, "internal data space is used (default)"},
-  {0,   OPTION_MEDIUM_MODEL, NULL, "external paged data space is used"},
-  {0,   OPTION_LARGE_MODEL, NULL, "external data space is used"},
-  {0,   OPTION_HUGE_MODEL, NULL, "functions are banked, data in external space"},
   {0,   "--stack-auto", &options.stackAuto, "Stack automatic variables"},
   {0,   "--xstack", &options.useXstack, "Use external stack"},
   {0,   "--int-long-reent", &options.intlong_rent, "Use reentrant calls on the int and long support functions"},
@@ -236,6 +237,7 @@ static const OPTION optionsTable[] = {
   {0,   OPTION_MAX_ALLOCS_PER_NODE, &options.max_allocs_per_node, "Maximum number of register assignments considered at each node of the tree decomposition", CLAT_INTEGER},
   {0,   OPTION_NO_LOSPRE, NULL, "Disable lospre"},
   {0,   OPTION_ALLOW_UNSAFE_READ, NULL, "Allow optimizations to read any memory location anytime"},
+  {0,   "--nostdlibcall", &optimize.noStdLibCall, "Disable optimization of calls to standard library"},
 
   {0,   NULL, NULL, "Internal debugging options"},
   {0,   OPTION_DUMP_AST, &options.dump_ast, "Dump front-end AST before generating i-code"},
@@ -325,6 +327,9 @@ static PORT *_ports[] = {
 #if !OPT_DISABLE_TLCS90
   &tlcs90_port,
 #endif
+#if !OPT_DISABLE_EZ80_Z80
+  &ez80_z80_port,
+#endif
 #if !OPT_DISABLE_AVR
   &avr_port,
 #endif
@@ -352,6 +357,15 @@ static PORT *_ports[] = {
 #if !OPT_DISABLE_STM8
   &stm8_port,
 #endif
+#if !OPT_DISABLE_PDK13
+  &pdk13_port,
+#endif
+#if !OPT_DISABLE_PDK14
+  &pdk14_port,
+#endif
+#if !OPT_DISABLE_PDK15
+  &pdk15_port,
+#endif
 };
 
 #define NUM_PORTS (sizeof(_ports)/sizeof(_ports[0]))
@@ -611,6 +625,7 @@ setDefaultOptions (void)
   options.std_c95 = 1;
   options.std_c99 = 1;
   options.std_c11 = 1;          /* default to C11 (we want inline by default, so we need at least C99, and support for C11 is more complete than C99) */
+  options.std_c2x = 0;
   options.code_seg = CODE_NAME ? Safe_strdup (CODE_NAME) : NULL;        /* default to CSEG for generated code */
   options.const_seg = CONST_NAME ? Safe_strdup (CONST_NAME) : NULL;     /* default to CONST for generated code */
   options.data_seg = DATA_NAME ? Safe_strdup (DATA_NAME) : NULL;        /* default to DATA for non-initialized data */
@@ -869,9 +884,6 @@ scanOptionsTable (const OPTION * optionsTable, char shortOpt, const char *longOp
                (optionsTable[i].arg_type == CLAT_BOOLEAN && len == strlen (longOpt) && optionsTable[i].longOpt)) &&
               strncmp (optionsTable[i].longOpt, longOpt, len) == 0)
             {
-              if (strncmp ("--nojtbound", longOpt, len) == 0)
-                werror (W_DEPRECATED_OPTION, "--nojtbound");
-
               /* If it is a flag then we can handle it here */
               if (optionsTable[i].pparameter != NULL)
                 {
@@ -1142,6 +1154,7 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 0;
               options.std_c99 = 0;
               options.std_c11 = 0;
+              options.std_c2x = 0;
               options.std_sdcc = 0;
               continue;
             }
@@ -1151,6 +1164,7 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 1;
               options.std_c99 = 0;
               options.std_c11 = 0;
+              options.std_c2x = 0;
               options.std_sdcc = 0;
               continue;
             }
@@ -1160,6 +1174,7 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 1;
               options.std_c99 = 1;
               options.std_c11 = 0;
+              options.std_c2x = 0;
               options.std_sdcc = 0;
               continue;
             }
@@ -1169,6 +1184,17 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 1;
               options.std_c99 = 1;
               options.std_c11 = 1;
+              options.std_c2x = 0;
+              options.std_sdcc = 0;
+              continue;
+            }
+
+          if (strcmp (argv[i], OPTION_STD_C2X) == 0)
+            {
+              options.std_c95 = 1;
+              options.std_c99 = 1;
+              options.std_c11 = 1;
+              options.std_c2x = 1;
               options.std_sdcc = 0;
               continue;
             }
@@ -1178,6 +1204,7 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 0;
               options.std_c99 = 0;
               options.std_c11 = 0;
+              options.std_c2x = 0;
               options.std_sdcc = 1;
               continue;
             }
@@ -1187,6 +1214,7 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 1;
               options.std_c99 = 1;
               options.std_c11 = 0;
+              options.std_c2x = 0;
               options.std_sdcc = 1;
               continue;
             }
@@ -1196,6 +1224,17 @@ parseCmdLine (int argc, char **argv)
               options.std_c95 = 1;
               options.std_c99 = 1;
               options.std_c11 = 1;
+              options.std_c2x = 0;
+              options.std_sdcc = 1;
+              continue;
+            }
+
+          if (strcmp (argv[i], OPTION_STD_SDCC2X) == 0)
+            {
+              options.std_c95 = 1;
+              options.std_c99 = 1;
+              options.std_c11 = 1;
+              options.std_c2x = 1;
               options.std_sdcc = 1;
               continue;
             }
@@ -2129,7 +2168,7 @@ preProcess (char **envp)
         since its name makes it non-compliant.
         It got removed a few times, but keeps coming back.
         This time it got added back for the 3.7.0 release
-        to support the old SiLabs */
+        to support the old SiLabs IDE */
       if (TARGET_IS_MCS51 && options.std_sdcc)
         {
           struct dbuf_s dbuf;
@@ -2157,7 +2196,7 @@ preProcess (char **envp)
       addSet (&preArgvSet, Safe_strdup ("-D__STDC_NO_ATOMICS__=1"));
       addSet (&preArgvSet, Safe_strdup ("-D__STDC_NO_VLA__=1"));
 
-      /* Character encoding */
+      /* Character encoding  - these need to be set in device/lib/Makefile.in for $CPP, too */
       addSet (&preArgvSet, Safe_strdup ("-D__STDC_ISO_10646__=201409L")); // wchar_t is UTF-32
       addSet (&preArgvSet, Safe_strdup ("-D__STDC_UTF_16__=1")); // char16_t is UTF-16
       addSet (&preArgvSet, Safe_strdup ("-D__STDC_UTF_32__=1")); // char32_t is UTF-32
