@@ -82,7 +82,7 @@ typedef struct bucket
 {
   void *sym;                    /* pointer to the object      */
   char name[SDCC_NAME_MAX + 1]; /* name of this symbol        */
-  int level;                    /* nest level for this symbol */
+  long level;                   /* nest level for this symbol */
   int block;                    /* belongs to which block     */
   struct bucket *prev;          /* ptr 2 previous bucket      */
   struct bucket *next;          /* ptr 2 next bucket          */
@@ -92,7 +92,7 @@ bucket;
 typedef struct structdef
 {
   char tag[SDCC_NAME_MAX + 1];  /* tag part of structure      */
-  unsigned char level;          /* Nesting level              */
+  long level;                   /* Nesting level              */
   int block;                    /* belongs to which block     */
   struct symbol *fields;        /* pointer to fields          */
   unsigned size;                /* sizeof the table in bytes  */
@@ -163,6 +163,7 @@ typedef struct specifier
   unsigned b_short:1;               /* 1=short int                */
   unsigned b_unsigned:1;            /* 1=unsigned, 0=signed       */
   unsigned b_signed:1;              /* just for sanity checks only*/
+  bool     b_implicit_sign:1;       /* signedness not explicitly specified - needed to keep char a separate type from signed char and unsigned char. */
   unsigned b_static:1;              /* 1=static keyword found     */
   unsigned b_extern:1;              /* 1=extern found             */
   unsigned b_inline:1;              /* inline function requested  */
@@ -274,13 +275,18 @@ typedef struct sym_link
     unsigned smallc:1;              /* Parameters on stack are passed in reverse order */
     unsigned z88dk_fastcall:1;      /* For the z80-related ports: Function has a single paramter of at most 32 bits that is passed in dehl */
     unsigned z88dk_callee:1;        /* Stack pointer adjustment for parameters passed on the stack is done by the callee */
-    unsigned intno;                 /* 1=Interrupt service routine          */
+    unsigned z88dk_shortcall:1;     /* Short call available via rst (see values later) (Z80 only) */
+    unsigned z88dk_has_params_offset:1;     /* Has a parameter offset (Z80 only) */
+    unsigned intno;                 /* Number of interrupt for interrupt service routine */
     short regbank;                  /* register bank 2b used                */
     unsigned builtin;               /* is a builtin function                */
     unsigned javaNative;            /* is a JavaNative Function (TININative ONLY) */
     unsigned overlay;               /* force parameters & locals into overlay segment */
     unsigned hasStackParms;         /* function has parameters on stack     */
     bool preserved_regs[9];         /* Registers preserved by the function - may be an underestimate */
+    unsigned char z88dk_shortcall_rst;  /* Rst for a short call */
+    unsigned short z88dk_shortcall_val; /* Value for a short call */
+    unsigned short z88dk_params_offset;  /* Additional offset from for arguments */
   } funcAttrs;
 
   struct sym_link *next;            /* next element on the chain  */
@@ -292,7 +298,7 @@ typedef struct symbol
   char name[SDCC_SYMNAME_MAX + 1];  /* Input Variable Name     */
   char rname[SDCC_NAME_MAX + 1];    /* internal name           */
 
-  short level;                      /* declaration lev,fld offset */
+  long level;                       /* declaration lev,fld offset */
   short block;                      /* sequential block # of definition */
   int seqPoint;                     /* sequence point defined or, if unbound, used */
   int key;
@@ -338,7 +344,8 @@ typedef struct symbol
   unsigned spildir:1;               /* spilt in direct space */
   unsigned ptrreg:1;                /* this symbol assigned to a ptr reg */
   unsigned noSpilLoc:1;             /* cannot be assigned a spil location */
-  unsigned div_flag_safe:1;         /* we know this function is safe to call with undocumented stm8 flag bit 6 set*/
+  bool funcDivFlagSafe:1;           /* we know this function is safe to call with undocumented stm8 flag bit 6 set*/
+  bool funcUsesVolatile:1;          /* The function accesses a volatile variable */
   unsigned isstrlit;                /* is a string literal and it's usage count  */
   unsigned accuse;                  /* can be left in the accumulator
                                        On the Z80 accuse is divided into
@@ -367,8 +374,8 @@ typedef struct symbol
     struct set *itmpStack;          /* symbols spilt @ this stack location */
   }
   usl;
-  signed char bitVar;               /* if bitVar != 0: this is a bit variable, bitVar is the size in bits */
-  char bitUnnamed:1;                /* unnamed bit variable */
+  int bitVar;                       /* if bitVar != 0: this is a bit variable, bitVar is the size in bits */
+  unsigned bitUnnamed:1;            /* unnamed bit variable */
   unsigned offset;                  /* offset from top if struct */
 
   int lineDef;                      /* defined line number        */
@@ -455,6 +462,8 @@ extern sym_link *validateLink (sym_link * l,
 #define IFFUNC_ISZ88DK_FASTCALL(x) (IS_FUNC(x) && FUNC_ISZ88DK_FASTCALL(x))
 #define FUNC_ISZ88DK_CALLEE(x) (x->funcAttrs.z88dk_callee)
 #define IFFUNC_ISZ88DK_CALLEE(x) (IS_FUNC(x) && FUNC_ISZ88DK_CALLEE(x))
+#define FUNC_ISZ88DK_SHORTCALL(x) (x->funcAttrs.z88dk_shortcall)
+#define IFFUNC_ISZ88DK_SHORTCALL(x) (IS_FUNC(x) && FUNC_ISZ88DK_SHORTCALL(x))
 
 #define BANKED_FUNCTIONS        ( options.model == MODEL_HUGE || \
                                   ( (options.model == MODEL_LARGE || options.model == MODEL_MEDIUM) && \
@@ -467,7 +476,7 @@ extern sym_link *validateLink (sym_link * l,
 #define SPEC_LONGLONG(x) validateLink(x, "SPEC_LONGLONG", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_longlong
 #define SPEC_SHORT(x) validateLink(x, "SPEC_LONG", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_short
 #define SPEC_USIGN(x) validateLink(x, "SPEC_USIGN", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_unsigned
-#define SPEC_SIGN(x) validateLink(x, "SPEC_USIGN", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_signed
+#define SPEC_SIGN(x) validateLink(x, "SPEC_SIGN", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_signed
 #define SPEC_SCLS(x) validateLink(x, "SPEC_SCLS", #x, SPECIFIER, __FILE__, __LINE__)->select.s.sclass
 #define SPEC_ENUM(x) validateLink(x, "SPEC_ENUM", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_isenum
 #define SPEC_OCLS(x) validateLink(x, "SPEC_OCLS", #x, SPECIFIER, __FILE__, __LINE__)->select.s.oclass
@@ -617,6 +626,8 @@ extern symbol *fp16x16conv[2][5][2];
 /* Dims: shift left/shift right, BYTE/WORD/DWORD/QWORD, SIGNED/UNSIGNED */
 extern symbol *rlrr[2][4][2];
 
+extern symbol *memcpy_builtin;
+
 #define SCHARTYPE       multypes[0][0]
 #define UCHARTYPE       multypes[0][1]
 #define INTTYPE         multypes[1][0]
@@ -644,7 +655,7 @@ typedef enum
 
 /* forward definitions for the symbol table related functions */
 void initSymt ();
-symbol *newSymbol (const char *, int);
+symbol *newSymbol (const char *, long);
 sym_link *newLink (SYM_LINK_CLASS);
 sym_link *newFloatLink ();
 structdef *newStruct (const char *);
@@ -658,7 +669,7 @@ symbol *copySymbol (const symbol *);
 symbol *copySymbolChain (const symbol *);
 void printSymChain (symbol *, int);
 void printStruct (structdef *, int);
-char *genSymName (int);
+char *genSymName (long);
 sym_link *getSpec (sym_link *);
 int compStructSize (int, structdef *);
 sym_link *copyLinkChain (const sym_link *);
@@ -676,10 +687,10 @@ sym_link *newLongLink ();
 sym_link *newBoolLink ();
 sym_link *newVoidLink ();
 int compareType (sym_link *, sym_link *);
-int compareTypeExact (sym_link *, sym_link *, int);
+int compareTypeExact (sym_link *, sym_link *, long);
 int compareTypeInexact (sym_link *, sym_link *);
 int checkFunction (symbol *, symbol *);
-void cleanUpLevel (bucket **, int);
+void cleanUpLevel (bucket **, long);
 void cleanUpBlock (bucket **, int);
 symbol *getAddrspace (sym_link *type);
 int funcInChain (sym_link *);
@@ -689,7 +700,7 @@ symbol *getStructElement (structdef *, symbol *);
 sym_link *computeType (sym_link *, sym_link *, RESULT_TYPE, int);
 void processFuncPtrArgs (sym_link *);
 void processFuncArgs (symbol *);
-int isSymbolEqual (symbol *, symbol *);
+int isSymbolEqual (const symbol *, const symbol *);
 int powof2 (TYPE_TARGET_ULONG);
 void dbuf_printTypeChain (sym_link *, struct dbuf_s *);
 void printTypeChain (sym_link *, FILE *);
@@ -700,11 +711,11 @@ void pointerTypes (sym_link *, sym_link *);
 void cdbStructBlock (int);
 void initHashT ();
 bucket *newBucket ();
-void addSym (bucket **, void *, char *, int, int, int checkType);
+void addSym (bucket **, void *, char *, long, int, int checkType);
 void deleteSym (bucket **, void *, const char *);
 void *findSym (bucket **, void *, const char *);
 void *findSymWithLevel (bucket **, struct symbol *);
-void *findSymWithBlock (bucket **, struct symbol *, int, int);
+void *findSymWithBlock (bucket **, struct symbol *, int, long);
 void changePointer (sym_link * p);
 void checkTypeSanity (sym_link * etype, const char *name);
 sym_link *typeFromStr (const char *);
