@@ -2,6 +2,7 @@
  */
 #include <testfwk.h>
 
+#ifndef __SDCC_pdk14
 char func(char a)
 {
   return a-1;
@@ -15,13 +16,16 @@ static struct
 } s;
 
 static char (* __xdata func_ptr) (char);
+#endif
 
 void
 testFptr(void)
 {
+#if !defined( __SDCC_pdk14) && !defined( __SDCC_pdk15)
   char b = 10;
   a      = 10;
   s.c    = 10;
+
   func_ptr=func;
   
   // works as expected
@@ -30,4 +34,6 @@ testFptr(void)
   
   // error passing s.c to func
   ASSERT(func_ptr(s.c) == 9);
+#endif
 }
+
